"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayRequestContainer = void 0;
const injection_1 = require("injection");
const container_1 = require("./container");
class MidwayRequestContainer extends container_1.MidwayContainer {
    constructor(applicationContext, ctx) {
        super();
        this.parent = applicationContext;
        this.applicationContext = applicationContext;
        if (ctx) {
            this.updateContext(ctx);
        }
    }
    updateContext(ctx) {
        // this.registry.clearAll();
        this.ctx = ctx;
        // register ctx
        this.registerObject('ctx', ctx);
        // register contextLogger
        this.registerObject('logger', ctx.logger);
    }
    get(identifier, args) {
        if (typeof identifier !== 'string') {
            identifier = this.getIdentifier(identifier);
        }
        if (this.registry.hasObject(identifier)) {
            return this.registry.getObject(identifier);
        }
        const definition = this.applicationContext.registry.getDefinition(identifier);
        if (definition && definition.isRequestScope()) {
            // create object from applicationContext definition for requestScope
            return this.getManagedResolverFactory().create(definition, args);
        }
        if (this.parent) {
            return this.parent.get(identifier, args);
        }
    }
    async getAsync(identifier, args) {
        if (typeof identifier !== 'string') {
            identifier = this.getIdentifier(identifier);
        }
        if (this.registry.hasObject(identifier)) {
            return this.registry.getObject(identifier);
        }
        const definition = this.applicationContext.registry.getDefinition(identifier);
        if (definition && definition.isRequestScope()) {
            if (definition.creator.constructor.name === 'FunctionWrapperCreator') {
                const valueManagedIns = new injection_1.ManagedValue(this, injection_1.VALUE_TYPE.OBJECT);
                definition.constructorArgs = [valueManagedIns];
            }
            // create object from applicationContext definition for requestScope
            return this.getManagedResolverFactory().createAsync(definition, args);
        }
        if (this.parent) {
            return this.parent.getAsync(identifier, args);
        }
    }
}
exports.MidwayRequestContainer = MidwayRequestContainer;
//# sourceMappingURL=requestContainer.js.map